@@ -1,6 +1,6 @@
 /*
     routing.c, manipulating routing table for PPTP Client
-    Copyright (C) 2006  James Cameron <quozl@us.netrek.org>
+    Copyright (C) 2006  Free Software Foundation
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -109,18 +109,27 @@ void routing_init(char *ip) {
       }
       break;
     }
-#else /* Solaris */ 
+#endif /* Solaris */ 
+#if defined(__linux)
   char buf[256];
+  char tbuf[256];
+  const char *uid;
   FILE *p;
 
   snprintf(buf, 255, "%s route get %s", IP_BINARY, ip);
   p = popen(buf, "r");
   fgets(buf, 255, p);
   /* TODO: check for failure of fgets */
-  route = strdup(buf);
+  uid = strstr(buf, " uid");
+  if (uid) {
+    snprintf(tbuf, uid - buf + 1, "%s", buf);
+    route = strdup(tbuf);
+  } else {
+    route = strdup(buf);
+  }
   pclose(p);
   /* TODO: check for failure of command */
-#endif /* Solaris */
+#endif /* __linux__ */
 }
 
 void routing_start(void) {
@@ -143,14 +152,15 @@ void routing_start(void) {
   if ( write(rts, &rtm, rtm.hdr.rtm_msglen) != rtm.hdr.rtm_msglen ) {
     log("Error adding route: %s", strerror(errno));
   }
-#else /* Solaris */
+#endif
+#if defined(__linux__)
   char buf[256];
   FILE *p;
 
   snprintf(buf, 255, "%s route replace %s", IP_BINARY, route);
   p = popen(buf, "r");
   pclose(p);
-#endif /* Solaris */
+#endif /* __linux__ */
 }
 
 void routing_end(void) {
@@ -173,12 +183,13 @@ void routing_end(void) {
   if ( write(rts, &rtm, rtm.hdr.rtm_msglen) != rtm.hdr.rtm_msglen ) {
     log("Error deleting route: %s", strerror(errno));
   }
-#else /* Solaris */
+#endif /* Solaris */
+#if defined(__linux__)
   char buf[256];
   FILE *p;
 
   snprintf(buf, 255, "%s route delete %s", IP_BINARY, route);
   p = popen(buf, "r");
   pclose(p);
-#endif /* Solaris */
+#endif /* __linux__ */
 }
